unit FileToPICClientThread;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, IdTCPClient, FileToPICSendFile, FileToPICUtils;

type

  TSyncStatus = class
  public
    FSender: TObject;
    FCallback: TNotifyEvent;
    procedure CallTheCalback;
  end;

  TSyncSendingFileProgress = class
    FPosition: DWord;
    FCallback: TOnTransferFileProgress;
    procedure CallTheCalback;
  end;

  TSyncAddToLog = class
    FMsg: string;
    FCallback: TOnAddMessageToLog;
    procedure CallTheCalback;
  end;

  TClientThread = class(TThread)
  private
    FClient: TIdTCPClient;
    FOnConnected: TNotifyEvent;
    FOnDisconnected: TNotifyEvent;
    FTerminateMessage: string;
    FFileToSend: string;
    FSendBeat: string;
    FBacklightSetting: Byte;
    FUpdateBacklightSetting: string; //use string, because it is automatically initialized to ''

    FCancelSendingFile: Boolean;
    FOnSendingFileProgress: TOnTransferFileProgress;
    FOnMessage: TOnAddMessageToLog;
    FOnFileTransferDone: TOnAddMessageToLog;

    FTriggerDisconnect: string;  //better than boolean

    FHost: string;
    FPort: Integer;
    FConnectTimeout: Integer;
    FReadTimeout: Integer;

    procedure IdTCPClientConnected(Sender: TObject);
    procedure IdTCPClientDisconnected(Sender: TObject);

    procedure SendingFileOnProgress(Position: DWord);
    procedure AddToLogFromThread(AMsg: string);

    procedure SendFileFromThread;
  protected
    procedure Execute; override;
  public
    procedure ConnectClient(AHost: string; APort: Word; AConnectTimeout, AReadTimeout: Integer);
    procedure DisconnectClient;
    function ClientConnected: Boolean;

    procedure SendFile(AFileName: string);
    procedure CancelSendingFile;
    procedure SendBeat;
    procedure SendBacklightSetting(ASetting: Byte);

    property OnConnected: TNotifyEvent read FOnConnected write FOnConnected;
    property OnDisconnected: TNotifyEvent read FOnDisconnected write FOnDisconnected;
    function GetHost: string;
    function GetPort: Word;

    property TerminateMessage: string read FTerminateMessage write FTerminateMessage;
    property OnSendingFileProgress: TOnTransferFileProgress read FOnSendingFileProgress write FOnSendingFileProgress;
    property OnMessage: TOnAddMessageToLog read FOnMessage write FOnMessage;
    property OnFileTransferDone: TOnAddMessageToLog read FOnFileTransferDone write FOnFileTransferDone;
  end;


implementation

uses
  FileToPICConsts, Dialogs, IdIOHandlerStack;


{ TSyncStatus }

procedure TSyncStatus.CallTheCalback;
begin
  FCallback(FSender);
end;


{ TSyncSendingFileProgress }

procedure TSyncSendingFileProgress.CallTheCalback;
begin
  FCallback(FPosition);
end;


{ TSyncAddToLog }

procedure TSyncAddToLog.CallTheCalback;
begin
  FCallback(FMsg);
end;

{ TClientThread }

procedure TClientThread.IdTCPClientConnected(Sender: TObject);
var
  ASyncObj: TSyncStatus;
begin
  if Assigned(FOnConnected) then
  begin
    ASyncObj := TSyncStatus.Create;
    try
      ASyncObj.FCallback := FOnConnected;
      ASyncObj.FSender := Sender;
      Self.Synchronize(@ASyncObj.CallTheCalback);
    finally
      ASyncObj.Free;
    end;
  end;
end;


procedure TClientThread.IdTCPClientDisconnected(Sender: TObject);
var
  ASyncObj: TSyncStatus;
begin
  if Assigned(FOnDisconnected) then
  begin
    ASyncObj := TSyncStatus.Create;
    try
      ASyncObj.FCallback := FOnDisconnected;
      ASyncObj.FSender := Sender;
      Self.Synchronize(@ASyncObj.CallTheCalback);
    finally
      ASyncObj.Free;
    end;
  end;
end;


procedure TClientThread.SendingFileOnProgress(Position: DWord);
var
  ASyncObj: TSyncSendingFileProgress;
begin
  if Assigned(FOnSendingFileProgress) then
  begin
    ASyncObj := TSyncSendingFileProgress.Create;
    try
      ASyncObj.FCallback := FOnSendingFileProgress;
      ASyncObj.FPosition := Position;
      Self.Synchronize(@ASyncObj.CallTheCalback);
    finally
      ASyncObj.Free;
    end;
  end;
end;


procedure TClientThread.AddToLogFromThread(AMsg: string);
var
  ASyncObj: TSyncAddToLog;
begin
  if Assigned(FOnMessage) then
  begin
    ASyncObj := TSyncAddToLog.Create;
    try
      ASyncObj.FCallback := FOnMessage;
      ASyncObj.FMsg := AMsg;
      Self.Synchronize(@ASyncObj.CallTheCalback);
    finally
      ASyncObj.Free;
    end;
  end;
end;


procedure TClientThread.ConnectClient(AHost: string; APort: Word; AConnectTimeout, AReadTimeout: Integer);
begin
  FHost := AHost;
  FPort := APort;
  FConnectTimeout := AConnectTimeout;
  FReadTimeout := AReadTimeout;
end;


procedure TClientThread.DisconnectClient;
begin
  if not Assigned(FClient) then
    Exit;

  FTriggerDisconnect := 'trigger';
end;


function TClientThread.ClientConnected: Boolean;
begin
  Result := Assigned(FClient) and FClient.Connected;
end;


procedure TClientThread.SendFileFromThread;
begin
  FTerminateMessage := SendFileToServer(FFileToSend, FClient, @AddToLogFromThread, @SendingFileOnProgress, @FCancelSendingFile);
end;


procedure TClientThread.SendFile(AFileName: string);
begin
  FCancelSendingFile := False;
  FFileToSend := AFileName;
end;


procedure TClientThread.CancelSendingFile;
begin
  FCancelSendingFile := True;
end;


procedure TClientThread.SendBeat;
begin
  FSendBeat := 'Beat'; //a string, different than ''
end;


procedure TClientThread.SendBacklightSetting(ASetting: Byte);
begin
  FBacklightSetting := ASetting;
  FUpdateBacklightSetting := 'Setting'; //a string, different than ''
end;


procedure SendBacklightSettingFromThread(AClient: TIdTCPClient; ASetting: Byte);
var
  AMemStream: TMemoryStream;
  DWordBuf: DWord;
  ByteBuf: Byte;
begin
  AMemStream := TMemoryStream.Create;
  try
    AMemStream.Write(CSendBacklightCmd[1], Length(CSendFileToSrvCmd)); //AMemStream.WriteAnsiString(CSendFileToSrvCmd + #13#10);
    ByteBuf := 13; AMemStream.Write(ByteBuf, 1); //AMemStream.WriteByte(13);
    ByteBuf := 10; AMemStream.Write(ByteBuf, 1); //AMemStream.WriteByte(10);

    DWordBuf := ASetting; AMemStream.Write(DWordBuf, 4); //AMemStream.WriteDWord(Length(AFileName));

    AMemStream.Position := 0;
    AClient.Socket.Write(AMemStream, AMemStream.Size);  //use a single write, for server compatibility :(
    Sleep(50); //wait a bit, since no feedback is expected from server
  finally
    AMemStream.Free;
  end;
end;


procedure TClientThread.Execute;
begin
  FTerminateMessage := '';

  try
    FClient := TIdTCPClient.Create(nil);
    try
      FClient.Host := FHost;
      FClient.Port := FPort;
      FClient.ConnectTimeout := FConnectTimeout;
      FClient.ReadTimeout := FReadTimeout;

      FClient.OnConnected := @IdTCPClientConnected;
      FClient.OnDisconnected := @IdTCPClientDisconnected;

      FClient.Connect;
      FClient.Socket.ConnectTimeout := FConnectTimeout;
      FClient.Socket.ReadTimeout := FReadTimeout;

      repeat
        if FFileToSend <> '' then
        begin
          FTerminateMessage := '';
          try
            SendFileFromThread;
          except
            on E: Exception do
            begin
              FTerminateMessage := E.Message;
              raise;
            end;
          end;
          FFileToSend := '';

          if Assigned(FOnFileTransferDone) then
            FOnFileTransferDone(FTerminateMessage);
        end;

        if FSendBeat <> '' then
        begin
          FClient.Socket.WriteLn(CSendBeatCmd);
          FSendBeat := '';
        end;

        if FUpdateBacklightSetting <> '' then
        begin
          SendBacklightSettingFromThread(FClient, FBacklightSetting);
          FUpdateBacklightSetting := '';
        end;

        if FTriggerDisconnect <> '' then
        begin
          FTriggerDisconnect := '';
          if FClient.Connected then
            FClient.Socket.WriteLn(CDisconnectCmd);

          FClient.Disconnect(True);
          Terminate;
        end;

        Sleep(1); //do not kill the CPU
      until Terminated;
    finally
      try
        if FClient.Connected then
          FClient.Disconnect(True);
      finally
        FreeAndNil(FClient);
      end;
    end;
  except
    on E: Exception do
    begin
      FTerminateMessage := E.Message;
      Terminate;
    end;
  end;
end;


function TClientThread.GetHost: string;
begin
  if Assigned(FClient) then
    Result := FClient.Host
  else
    Result := '';
end;


function TClientThread.GetPort: Word;
begin
  if Assigned(FClient) then
    Result := FClient.Port
  else
    Result := 0;
end;

end.

