unit FileToPICMainForm;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  ComCtrls, IdIOHandlerStack, FileToPICClientThread;

type
  { TfrmFileToPICMain }

  TfrmFileToPICMain = class(TForm)
    btnConnect: TButton;
    btnDisconnect: TButton;
    btnSendFile: TButton;
    btnCancelSendingFile: TButton;
    btnSendBeat: TButton;
    btnClearLog: TButton;
    chkEnableBeat5000: TCheckBox;
    cmbHost: TComboBox;
    grpConnection: TGroupBox;
    lblBacklight: TLabel;
    lblHost: TLabel;
    lbeConnectTimeout: TLabeledEdit;
    lbeReadTimeout: TLabeledEdit;
    lbePort: TLabeledEdit;
    memLog: TMemo;
    OpenDialog1: TOpenDialog;
    pnlProgress: TPanel;
    pnlConnectionStatus: TPanel;
    pnlBeat: TPanel;
    prbSendFileProgress: TProgressBar;
    prbConnectTimeout: TProgressBar;
    tmrStartup: TTimer;
    tmrConnectTimeout: TTimer;
    tmrSwitchOffBeat: TTimer;
    tmrBeat5000: TTimer;
    trbBackLight: TTrackBar;
    procedure btnCancelSendingFileClick(Sender: TObject);
    procedure btnClearLogClick(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure btnDisconnectClick(Sender: TObject);
    procedure btnSendBeatClick(Sender: TObject);
    procedure btnSendFileClick(Sender: TObject);
    procedure chkEnableBeat5000Change(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure tmrBeat5000Timer(Sender: TObject);
    procedure tmrConnectTimeoutTimer(Sender: TObject);
    procedure tmrStartupTimer(Sender: TObject);
    procedure tmrSwitchOffBeatTimer(Sender: TObject);
    procedure trbBackLightChange(Sender: TObject);
  private
    FClientTh: TClientThread;
    FFileSizeToSend: DWord;

    procedure LoadSettings;
    procedure SaveSettings;
    procedure AddHostToList;

    procedure CreateClient(AHost: string; APort: Word; AConnectTimeout, AReadTimeout: Integer);
    procedure DisconnectClient;
    procedure ClientThreadOnTerminated(Sender: TObject);

    procedure ClientConnected(Sender: TObject);
    procedure ClientDisconnected(Sender: TObject);
    procedure SendingFileOnProgress(APosition: DWord);
    procedure ThreadOnAddMessageToLog(AMsg: string);
    procedure ThreadOnFileTransferDone(AMsg: string);

    procedure SendFile(AFileName: string);
    procedure CancelSendingFile;

    procedure SendBeat(ShowErrMsg: Boolean = True);
    procedure SendBacklightSetting(Setting: Byte);

    procedure SetStatusToConnected;
    procedure SetStatusToConnecting;
    procedure SetStatusToDisconnected;

    procedure EnableConnectionControls;
    procedure DisableConnectionControls;

    procedure AddLineToLog(s: string);

    procedure CustomExceptionHandler(Sender: TObject; E: Exception);
  public

  end;

var
  frmFileToPICMain: TfrmFileToPICMain;

implementation

{$R *.frm}

uses
  FileToPICUtils, IniFiles;

//See CatchUnhandledException and DumpExceptionCallStack in FreePascal docs  https://wiki.freepascal.org/Logging_exceptions
{procedure CatchUnhandledException(Obj: TObject; Addr: Pointer; FrameCount: Longint; Frames: PPointer);
var
  Message: string;
  i: LongInt;
  hstdout: ^Text;
begin
  hstdout := @stdout;
  Writeln(hstdout^, 'An unhandled exception occurred at $', sysBackTraceStr(addr), ' :');
  if Obj is exception then
  begin
    Message := Exception(Obj).ClassName + ' : ' + Exception(Obj).Message;
    Writeln(hstdout^, Message);
  end
  else
    Writeln(hstdout^, 'Exception object ', Obj.ClassName, ' is not of class Exception.');

  Writeln(hstdout^, BackTraceStrFunc(Addr));

  if (FrameCount > 0) then
  begin
    for i := 0 to FrameCount - 1 do
      Writeln(hstdout^, BackTraceStrFunc(Frames[i]));
  end;

  Writeln(hstdout^,'');
end;}


procedure DumpExceptionCallStack(E: Exception);
var
  I: Integer;
  Frames: PPointer;
  Report: string;
begin
  Report := 'Program exception! ' + LineEnding + 'Stacktrace:' + LineEnding + LineEnding;

  if E <> nil then
  begin
    Report := Report + 'Exception class: ' + E.ClassName + LineEnding + 'Message: ' + E.Message + LineEnding;
  end;

  Report := Report + BackTraceStrFunc(ExceptAddr);
  Frames := ExceptFrames;

  for I := 0 to ExceptFrameCount - 1 do
    Report := Report + LineEnding + BackTraceStrFunc(Frames[I]);

  frmFileToPICMain.AddLineToLog(Report);
end;


{ TfrmFileToPICMain }


procedure TfrmFileToPICMain.LoadSettings;
var
  Ini: TMemIniFile;
  i: Integer;
begin
  Ini := TMemIniFile.Create(ExtractFilePath(ParamStr(0)) + 'ClientSettings.ini');
  try
    Left := Ini.ReadInteger('Window', 'Left', 100);
    Top := Ini.ReadInteger('Window', 'Top', 100);
    Width := Ini.ReadInteger('Window', 'Width', Width);
    Height := Ini.ReadInteger('Window', 'Height', Height);

    cmbHost.Clear;
    for i := 0 to Ini.ReadInteger('Connection', 'HostCount', 0) - 1 do
      cmbHost.Items.Add(Ini.ReadString('Connection', 'Host_' + IntToStr(i), ''));

    cmbHost.Text := Ini.ReadString('Connection', 'SelectedHost', '');

    lbePort.Text := Ini.ReadString('Connection', 'Port', '7000');
    lbeConnectTimeout.Text := Ini.ReadString('Connection', 'ConnectTimeout', '20000');
    lbeReadTimeout.Text := Ini.ReadString('Connection', 'ReadTimeout', '10000');
  finally
    Ini.Free;
  end;
end;


procedure TfrmFileToPICMain.SaveSettings;
var
  Ini: TMemIniFile;
  i: Integer;
begin
  Ini := TMemIniFile.Create(ExtractFilePath(ParamStr(0)) + 'ClientSettings.ini');
  try
    Ini.WriteInteger('Window', 'Left', Left);
    Ini.WriteInteger('Window', 'Top', Top);
    Ini.WriteInteger('Window', 'Width', Width);
    Ini.WriteInteger('Window', 'Height', Height);

    Ini.WriteInteger('Connection', 'HostCount', cmbHost.Items.Count);
    for i := 0 to cmbHost.Items.Count - 1 do
      Ini.WriteString('Connection', 'Host_' + IntToStr(i), cmbHost.Items.Strings[i]);

    Ini.WriteString('Connection', 'SelectedHost', cmbHost.Text);

    Ini.WriteInteger('Connection', 'Port', StrToIntDef(lbePort.Text, 7000));
    Ini.WriteInteger('Connection', 'ConnectTimeout', StrToIntDef(lbeConnectTimeout.Text, 20000));
    Ini.WriteInteger('Connection', 'ReadTimeout', StrToIntDef(lbeReadTimeout.Text, 10000));
  finally
    Ini.Free;
  end;
end;


procedure TfrmFileToPICMain.AddHostToList;
begin
  if cmbHost.Items.IndexOf(cmbHost.Text) = -1 then
    cmbHost.Items.Add(cmbHost.Text);
end;


procedure TfrmFileToPICMain.CustomExceptionHandler(Sender: TObject; E: Exception);
begin
  DumpExceptionCallStack(E);
end;


procedure TfrmFileToPICMain.FormCreate(Sender: TObject);
begin
  FClientTh := nil;
  FFileSizeToSend := 0;

  Application.OnException := @CustomExceptionHandler;
  tmrStartup.Enabled := True;
end;


procedure TfrmFileToPICMain.tmrBeat5000Timer(Sender: TObject);
begin
  SendBeat(False);

  if not Assigned(FClientTh) then
    Exit;

  chkEnableBeat5000.Hint := 'Enable this to send a beat every 5s.' + #13#10 + 'Client status: ' + BoolToStr(FClientTh.ClientConnected, 'Connected', 'Not connected');
end;


procedure TfrmFileToPICMain.tmrConnectTimeoutTimer(Sender: TObject);
begin
  prbConnectTimeout.Position := prbConnectTimeout.Position + 1000;
  if prbConnectTimeout.Position = prbConnectTimeout.Max then
  begin
    tmrConnectTimeout.Enabled := False;
    prbConnectTimeout.Hide;
  end;
end;


procedure TfrmFileToPICMain.tmrStartupTimer(Sender: TObject);
begin
  tmrStartup.Enabled := False;
  LoadSettings;
end;


procedure TfrmFileToPICMain.tmrSwitchOffBeatTimer(Sender: TObject);
begin
  tmrSwitchOffBeat.Enabled := False;

  if Assigned(FClientTh) then
    pnlBeat.Color := clGreen
  else
    pnlBeat.Color := clOlive;

  pnlBeat.Font.Color := clWhite;
end;


procedure TfrmFileToPICMain.trbBackLightChange(Sender: TObject);
begin
  SendBacklightSetting(trbBackLight.Position);
  Sleep(10);
end;


procedure TfrmFileToPICMain.EnableConnectionControls;
begin
  btnConnect.Enabled := True;
  btnDisconnect.Enabled := False;     //disable diconnect button
  cmbHost.Enabled := True;
  lbePort.Enabled := True;
  lbeConnectTimeout.Enabled := True;
  lbeReadTimeout.Enabled := True;
  btnSendFile.Enabled := False;
  btnCancelSendingFile.Enabled := False;
  btnSendBeat.Enabled := False;

  prbConnectTimeout.Hide;
  tmrConnectTimeout.Enabled := False; //disable timer
end;


procedure TfrmFileToPICMain.DisableConnectionControls;
begin
  btnConnect.Enabled := False;
  btnDisconnect.Enabled := True;
  cmbHost.Enabled := False;
  lbePort.Enabled := False;
  lbeConnectTimeout.Enabled := False;
  lbeReadTimeout.Enabled := False;
  btnSendFile.Enabled := True;
  btnCancelSendingFile.Enabled := True;
  btnSendBeat.Enabled := True;

  prbConnectTimeout.Max := StrToIntDef(lbeConnectTimeout.Text, 20000);
  prbConnectTimeout.Position := 0;
  prbConnectTimeout.Show;
  tmrConnectTimeout.Enabled := True;
end;


procedure TfrmFileToPICMain.btnConnectClick(Sender: TObject);
begin
  SetStatusToConnecting;
  CreateClient(cmbHost.Text, StrToIntDef(lbePort.Text, 7000), StrToIntDef(lbeConnectTimeout.Text, 20000), StrToIntDef(lbeReadTimeout.Text, 10000));
  DisableConnectionControls;
  AddHostToList;
end;


procedure TfrmFileToPICMain.btnDisconnectClick(Sender: TObject);
begin
  DisconnectClient;
  EnableConnectionControls;
  SetStatusToDisconnected;
end;


procedure TfrmFileToPICMain.btnSendBeatClick(Sender: TObject);
begin
  SendBeat;
end;


procedure TfrmFileToPICMain.CreateClient(AHost: string; APort: Word; AConnectTimeout, AReadTimeout: Integer);
begin
  FClientTh := TClientThread.Create(True);
  FClientTh.OnConnected := @ClientConnected;
  FClientTh.OnDisconnected := @ClientDisconnected;
  FClientTh.OnTerminate := @ClientThreadOnTerminated;
  FClientTh.OnSendingFileProgress := @SendingFileOnProgress;
  FClientTh.OnMessage := @ThreadOnAddMessageToLog;
  FClientTh.OnFileTransferDone := @ThreadOnFileTransferDone;
  FClientTh.FreeOnTerminate := True;
  FClientTh.TerminateMessage := '';

  FClientTh.ConnectClient(AHost, APort, AConnectTimeout, AReadTimeout);
  FClientTh.Start;
end;


procedure TfrmFileToPICMain.DisconnectClient;
begin
  FClientTh.DisconnectClient;
end;


procedure TfrmFileToPICMain.SetStatusToConnected;
begin
  pnlConnectionStatus.Color := clLime;
  pnlConnectionStatus.Font.Color := clBlack;
  pnlConnectionStatus.Caption := 'Connected';
end;


procedure TfrmFileToPICMain.SetStatusToConnecting;
begin
  pnlConnectionStatus.Color := clYellow;
  pnlConnectionStatus.Font.Color := clBlack;
  pnlConnectionStatus.Caption := 'Connecting';
end;


procedure TfrmFileToPICMain.SetStatusToDisconnected;
begin
  pnlConnectionStatus.Color := clGreen;
  pnlConnectionStatus.Font.Color := clWhite;
  pnlConnectionStatus.Caption := 'Disconnected';
end;


procedure TfrmFileToPICMain.ClientConnected(Sender: TObject);
begin
  SetStatusToConnected;
  AddLineToLog('Connected to ' + FClientTh.GetHost + ':' + IntToStr(FClientTh.GetPort));
  prbConnectTimeout.Hide;
  tmrConnectTimeout.Enabled := False; //disable timer
end;


procedure TfrmFileToPICMain.ClientDisconnected(Sender: TObject);
begin
  SetStatusToDisconnected;
  AddLineToLog('Disconnected from ' + FClientTh.GetHost + ':' + IntToStr(FClientTh.GetPort));
end;


procedure TfrmFileToPICMain.btnSendFileClick(Sender: TObject);
begin
  if not Assigned(FClientTh) then
  begin
    AddLineToLog('Cannot send file. Not connected to server.');
    Exit;
  end;

  if not OpenDialog1.Execute then
    Exit;

  if chkEnableBeat5000.Checked then
  begin
    chkEnableBeat5000.Checked := False;
    chkEnableBeat5000.Enabled := False;
    btnSendFile.Tag := 1;
  end;

  SendFile(OpenDialog1.FileName);
  btnSendFile.Enabled := False;
  btnCancelSendingFile.Enabled := True;
end;


procedure TfrmFileToPICMain.chkEnableBeat5000Change(Sender: TObject);
begin
  tmrBeat5000.Enabled := chkEnableBeat5000.Checked;

  if not chkEnableBeat5000.Checked then
    chkEnableBeat5000.Hint := 'Enable this to show status.';
end;


procedure TfrmFileToPICMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  try
    SaveSettings;
  except
    on E: Exception do
      MessageDlg(Caption, 'Exception on saving settings: ' + E.Message, mtError, [mbOK], 0);
  end;
end;


procedure TfrmFileToPICMain.btnCancelSendingFileClick(Sender: TObject);
begin
  if not Assigned(FClientTh) then
    Exit;

  CancelSendingFile;
  btnSendFile.Enabled := True;
  btnCancelSendingFile.Enabled := False;

  if btnSendFile.Tag = 1 then
    chkEnableBeat5000.Checked := True;

  chkEnableBeat5000.Enabled := True;
end;


procedure TfrmFileToPICMain.btnClearLogClick(Sender: TObject);
begin
  if MessageDlg(Caption, 'Are you sure you want to clear the log?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    memLog.Clear;
end;


procedure TfrmFileToPICMain.SendingFileOnProgress(APosition: DWord);
begin
  if prbSendFileProgress.Min = 0 then
    prbSendFileProgress.Position := APosition
  else
    prbSendFileProgress.Position := APosition - MaxInt - 1;

  pnlProgress.Caption := IntToStr(APosition);
end;


procedure TfrmFileToPICMain.ThreadOnAddMessageToLog(AMsg: string);
begin
  AddLineToLog(AMsg);
end;


procedure TfrmFileToPICMain.ThreadOnFileTransferDone(AMsg: string);
begin
  AddLineToLog('Done sending file: ' + OpenDialog1.FileName);
  AddLineToLog(AMsg);

  if btnSendFile.Tag = 1 then
    chkEnableBeat5000.Checked := True;

  chkEnableBeat5000.Enabled := True;
  btnSendFile.Enabled := True;
  btnCancelSendingFile.Enabled := False;
end;


procedure TfrmFileToPICMain.ClientThreadOnTerminated(Sender: TObject);
begin
  SetStatusToDisconnected;
  AddLineToLog('Disconnected from ' + FClientTh.GetHost + ':' + IntToStr(FClientTh.GetPort) + '   Exit message: ' + FClientTh.TerminateMessage);

  FreeAndNil(FClientTh);
  EnableConnectionControls;
end;


procedure TfrmFileToPICMain.AddLineToLog(s: string);
var
  NowStr: string;
begin
  NowStr := DateTimeToStr(Now);
  s := StringReplace(s, #0, '#0', [rfReplaceAll]);
  s := StringReplace(s, #13#10, '  ', [rfReplaceAll]);
  s := StringReplace(s, #10, '  ', [rfReplaceAll]);

  memLog.Lines.Add(NowStr + '  ' + s);
end;


procedure TfrmFileToPICMain.SendFile(AFileName: string);
var
  ASize: Int64;
begin
  ASize := GetFileSizeOfSendingFile(AFileName);
  if ASize > $FFFFFFFF then
  begin
    MessageDlg(Caption, 'Files bigger than 4GB are not supported.', mtError, [mbOK], 0);
    Exit;
  end;

  if ASize > MaxInt then
  begin
    prbSendFileProgress.Min := -MaxInt - 1;
    prbSendFileProgress.Max := ASize - MaxInt - 1;
  end
  else
  begin
    prbSendFileProgress.Min := 0;
    prbSendFileProgress.Max := ASize;
  end;

  FFileSizeToSend := ASize;
  AddLineToLog('Sending file: ' + AFileName + '  of size: ' + IntToStr(ASize) + 'B');
  FClientTh.SendFile(AFileName);
end;


procedure TfrmFileToPICMain.CancelSendingFile;
begin
  FClientTh.CancelSendingFile;
  AddLineToLog('Cancelled sending file: ' + OpenDialog1.FileName);
end;


procedure TfrmFileToPICMain.SendBeat(ShowErrMsg: Boolean = True);
begin
  if not Assigned(FClientTh) then
  begin
    if ShowErrMsg then
      AddLineToLog('Client not connected');

    pnlBeat.Color := clYellow;
    pnlBeat.Font.Color := clBlack;
    tmrSwitchOffBeat.Enabled := True;

    Exit;
  end;

  try
    FClientTh.SendBeat;
  except
    if ShowErrMsg then
      AddLineToLog('Client not connected when sending beat.');
  end;

  pnlBeat.Color := clLime;
  pnlBeat.Font.Color := clBlack;
  tmrSwitchOffBeat.Enabled := True;
end;


procedure TfrmFileToPICMain.SendBacklightSetting(Setting: Byte);
begin
  if not Assigned(FClientTh) then
  begin
    AddLineToLog('Client not connected');
    Exit;
  end;

  try
    FClientTh.SendBacklightSetting(Setting);
  except
    AddLineToLog('Client not connected when setting backlight.');
  end;
end;


end.

