unit FileToPICSendFile;

{$IFDEF FPC}
  {$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils, IdTCPClient, FileToPICUtils;


function SendFileToServer(AFileName: string; AClient: TIdTCPClient; AddToLog: TOnAddMessageToLog; SendingFileOnProgress: TOnTransferFileProgress; CancelSendingFile: PBoolean): string;


implementation


uses
  FileToPICConsts, IdExceptionCore;


procedure ReadLnFromServerToClearBuffer(AClient: TIdTCPClient; AddToLog: TOnAddMessageToLog);
var
  j: Integer;
  TempStr: string;
  TempTimeout: Integer;
begin
  try
    AddToLog('..... Clearing srv tx buffer');

    TempStr := '';
    try
      TempTimeout := AClient.Socket.ReadTimeout;
      try
        AClient.Socket.ReadTimeout := 2000;
        repeat
          TempStr := TempStr + AClient.Socket.ReadChar;
        until Pos(#13#10, TempStr) > 0;
      finally
        AClient.Socket.ReadTimeout := TempTimeout;
      end;
    finally
      for j := 0 to 32 do
        TempStr := StringReplace(TempStr, Chr(j), '#' + IntToStr(j), [rfReplaceAll]);

      AddToLog('..... (hopefully) cleared srv tx buffer    Read: ' + TempStr);
    end;
  except
    on E: EIdReadTimeout do
    begin
    end
    else
      AddToLog('.................. Exception on clearing srv tx buffer: ' + AClient.Socket.InputBufferAsString);
  end;
end;


function SendFileToServer(AFileName: string; AClient: TIdTCPClient; AddToLog: TOnAddMessageToLog; SendingFileOnProgress: TOnTransferFileProgress; CancelSendingFile: PBoolean): string;
const
  CBufferSizePower = 10; //10;    //used as 2^CBufferSizePower      for values greater than 11, please modify the MCU code to increase the buffer size (FilePieceBuffer var)
var
  AFileStream: TFileStream;
  i, j: Integer;
  SentAmount, PieceCount, PieceSize: DWord;
  FileTransferResponse: string;
  TempStr: string;
  AMemStream: TMemoryStream;
  ExpectedResponse: string;
  ByteBuf: Byte;
  DWordBuf: DWord;
begin
  for i := 1 to 7 do
  begin
    AClient.Socket.WriteLn(CSendBeatCmd);
    Sleep(10);
  end;

  AFileStream := TFileStream.Create(AFileName, fmOpenRead + fmShareDenyWrite);
  try
    AFileName := ExtractFileName(AFileName);

    ReadLnFromServerToClearBuffer(AClient, AddToLog);

    AddToLog('.....' + CSendFileToSrvCmd);
    //TempStr := AFileName;

    AMemStream := TMemoryStream.Create;
    try
      AMemStream.Write(CSendFileToSrvCmd[1], Length(CSendFileToSrvCmd)); //AMemStream.WriteAnsiString(CSendFileToSrvCmd + #13#10);
      ByteBuf := 13; AMemStream.Write(ByteBuf, 1); //AMemStream.WriteByte(13);
      ByteBuf := 10; AMemStream.Write(ByteBuf, 1); //AMemStream.WriteByte(10);

      DWordBuf := Length(AFileName); AMemStream.Write(DWordBuf, 4); //AMemStream.WriteDWord(Length(AFileName));
      AMemStream.Write(AFileName[1], Length(AFileName)); //AMemStream.WriteAnsiString(FFileToSend);

      SetLength(TempStr, AMemStream.Size);
      AMemStream.Position := 0;
      AMemStream.Read(TempStr[1], AMemStream.Size);

      AMemStream.Position := 0;
      AClient.Socket.Write(AMemStream, AMemStream.Size);  //use a single write, for server compatibility :(
    finally
      AMemStream.Free;
    end;


    for i := 0 to 32 do
      TempStr := StringReplace(TempStr, Chr(i), '#' + IntToStr(i), [rfReplaceAll]);
    AddToLog('..... Header: ' + TempStr);

    FileTransferResponse := AClient.Socket.ReadLn(#13#10);
    for i := 0 to 32 do
      FileTransferResponse := StringReplace(FileTransferResponse, Chr(i), '#' + IntToStr(i), [rfReplaceAll]);
    AddToLog('..... FileTransferResponse = ' + FileTransferResponse);

    if FileTransferResponse = COKResponse then
    begin
      PieceSize := 1 shl CBufferSizePower;

      AddToLog('..... FileSize: ' + IntToStr(DWord(AFileStream.Size)));
      //FClient.Socket.Write(DWord(AFileStream.Size), False);

      AddToLog('..... CBufferSizePower: ' + IntToStr(Byte(CBufferSizePower)) + '  PieceSize: ' + IntToStr(PieceSize));
      //FClient.Socket.Write(Byte(CBufferSizePower), False);          //the server should stop reading from connection when this byte is 0 and delete the file


      //FClient.Socket.Write(DWordToBytesStr(AFileStream.Size) + Chr(Ord(CBufferSizePower)));
      AMemStream := TMemoryStream.Create;
      try
        DWordBuf := AFileStream.Size; AMemStream.Write(DWordBuf, 4); //AMemStream.WriteDWord(AFileStream.Size);
        ByteBuf := CBufferSizePower; AMemStream.Write(ByteBuf, 1); //AMemStream.WriteByte(CBufferSizePower);
        AMemStream.Position := 0;
        AClient.Socket.Write(AMemStream, AMemStream.Size);
      finally
        AMemStream.Free;
      end;

      SentAmount := 0;
      PieceCount := AFileStream.Size shr CBufferSizePower;
      AddToLog('..... PieceCount: ' + IntToStr(PieceCount));
      AFileStream.Position := 0;

      AddToLog('..... from CFSM_SStartReceivingFile   Expected: ' + CRequestNextPiece_Start + '    Read: ' + AClient.Socket.ReadLn(#13#10));

      for i := 0 to PieceCount - 1 do
      begin
        //AddToLog('..... piece: ' + IntToStr(i));

        AMemStream := TMemoryStream.Create;
        try
          if (CancelSendingFile <> nil) and CancelSendingFile^ then
          begin
            AClient.Socket.Write(Byte(CStopFileTransfer));
            Exit;
          end;
          //else
            //FClient.Socket.Write(Byte(CContinueFileTransfer), False);

          //FClient.Socket.Write(AFileStream, PieceSize);
          ByteBuf := CContinueFileTransfer; AMemStream.Write(ByteBuf, 1); //AMemStream.WriteByte(CContinueFileTransfer);
          DWordBuf := i; AMemStream.Write(DWordBuf, 4); //AMemStream.WriteDWord(i); //send also piece index  (useful for debugging) - also make sure the MCU reads it
          AMemStream.CopyFrom(AFileStream, PieceSize);

          AMemStream.Position := 0;
          AClient.Socket.Write(AMemStream, AMemStream.Size);
        finally
          AMemStream.Free;
        end;
        Inc(SentAmount, PieceSize);

        if Assigned(SendingFileOnProgress) then
          SendingFileOnProgress(SentAmount);

        Sleep(1);

        TempStr := '';
        ExpectedResponse := IntToStr(i) + CRequestNextPiece_Middle;
        try
          try
            TempStr := AClient.Socket.ReadLn(#13#10);
          finally
            for j := 0 to 32 do
              TempStr := StringReplace(TempStr, Chr(j), '#' + IntToStr(j), [rfReplaceAll]);

            //AddToLog('..... from CFSM_SReceiveFile   Expected: ' + ExpectedResponse + '    Read: ' + TempStr);  ///////// uncomment this to log every piece
          end;
        except
          on E: Exception do
          begin
            AddToLog('.................. Exception from CFSM_SReceiveFile    E: ' + E.Message);
            raise;
          end;
        end;

        if TempStr <> ExpectedResponse then
        begin
          Result := TempStr;
          AddToLog('..... Transfer got out of sync. Stopping...   Expected: ' + ExpectedResponse + '    Read: ' + TempStr);
          raise Exception.Create('Transfer got out of sync.');
          //Exit;
        end;
      end; //for

      AMemStream := TMemoryStream.Create;
      try
        if (CancelSendingFile <> nil) and CancelSendingFile^ then
        begin
          AClient.Socket.Write(Byte(CStopFileTransfer));
          Exit;
        end;

        AClient.Socket.Write(Byte(CFinishFileTransfer));
        Sleep(100);
        //AMemStream.WriteByte(CFinishFileTransfer);

        SentAmount := AFileStream.Size - SentAmount;
        DWordBuf := SentAmount; AMemStream.Write(DWordBuf, 4); //AMemStream.WriteDWord(SentAmount);  // send the last piece size

        if SentAmount > 0 then
        begin
          DWordBuf := PieceCount; AMemStream.Write(DWordBuf, 4); //AMemStream.WriteDWord(PieceCount); //send also piece index  (useful for debugging) - also make sure the MCU reads it
          AMemStream.CopyFrom(AFileStream, SentAmount);
          if Assigned(SendingFileOnProgress) then
            SendingFileOnProgress(AFileStream.Size);
        end;

        AMemStream.Position := 0;
        AClient.Socket.Write(AMemStream, AMemStream.Size);
        AddToLog('..... Last piece size : ' + IntToStr(SentAmount) + '  Last packet size: ' + IntToStr(AMemStream.Size));
      finally
        AMemStream.Free;
      end;

      AddToLog('..... from CFSM_SFinishReceivingFile   Expected: ' + CRequestNextPiece_End + '    Read: ' + AClient.Socket.ReadLn(#13#10, 10000));

      Result := COKResponse;
    end
    else
      if FileTransferResponse = CFAT32ErrResponse then
      begin
        AddToLog('..... ' + FileTransferResponse + ' .  Make sure the SD card (or USB drive) is connected and properly initialized. Also, the file should be writable.');
        Result := CFAT32ErrResponse;
      end
      else
      begin
        AddToLog('..... Terminating file transfer, because response from server is unknown: ' + StringReplace(FileTransferResponse, #0, '#0', [rfReplaceAll]));
        Result := CUnknownResponse;
      end;
  finally
    AFileStream.Free;
  end;
end;

end.

