unit FileToPICReceiveFile;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, IdContext, FileToPICUtils;

procedure ReceiveFileFromClient(ABaseDir: string; AContext: TIdContext; AddToLog: TOnAddMessageToLog; ReceivingFileOnProgress: TOnTransferFileProgress; ReceivingFileSize: TOnTransferFileSize);


implementation

uses
  FileToPICConsts;


procedure ReceiveFileFromClient(ABaseDir: string; AContext: TIdContext; AddToLog: TOnAddMessageToLog; ReceivingFileOnProgress: TOnTransferFileProgress; ReceivingFileSize: TOnTransferFileSize);
var
  FileName: string;
  FileNameLength: Integer;
  AFileStream: TFileStream;
  AMemStream: TMemoryStream;
  BufferSizePower: Byte;
  PieceCount, PieceIndex, PieceSize: DWord;
  LastPieceSize, ReceivedLastPieceSize, ExpectedFileSize: DWord;
  SentAmount: DWord;
  PieceState: Byte;
  i: Integer;
  ExpectedResponse: string;
begin
  FileNameLength := AContext.Connection.Socket.ReadUInt32(False);
  SetLength(FileName, FileNameLength);
  FileName := AContext.Connection.Socket.ReadString(FileNameLength);

  AddToLog('..... Receiving file: ' + FileName);

  try
    AFileStream := TFileStream.Create(ABaseDir + PathDelim + FileName, fmCreate + fmOpenWrite, fmShareDenyWrite);
    AContext.Connection.Socket.WriteLn(COKResponse);
  except
    AContext.Connection.Socket.WriteLn(CFAT32ErrResponse);
    Exit;
  end;

  try
    ExpectedFileSize := AContext.Connection.Socket.ReadUInt32(False);
    AFileStream.Position := 0;

    if Assigned(ReceivingFileOnProgress) then
      ReceivingFileOnProgress(0);

    if Assigned(ReceivingFileSize) then
      ReceivingFileSize(ExpectedFileSize);

    BufferSizePower := AContext.Connection.Socket.ReadByte;

    PieceCount := ExpectedFileSize shr BufferSizePower;
    AddToLog('..... PieceCount: ' + IntToStr(PieceCount));

    AContext.Connection.Socket.WriteLn(CRequestNextPiece_Start);

    PieceSize := 1 shl BufferSizePower;
    SentAmount := 0;

    for i := 0 to PieceCount do  //without "- 1", to allow processing the last piece
    begin
      AddToLog('..... piece: ' + IntToStr(i));

      PieceState := AContext.Connection.Socket.ReadByte;

      case PieceState of
        CStopFileTransfer:
        begin
          AddToLog('..... Stopping file transfer on demand.');
          Exit;
        end;

        CContinueFileTransfer:
        begin
          PieceIndex := AContext.Connection.Socket.ReadUInt32(False);
          if PieceIndex <> DWord(i) then
            AddToLog('..... transfer out of sync')
          else
            AddToLog('..... Receiving piece ' + IntToStr(PieceIndex));

          {AMemStream := TMemoryStream.Create;
          try
            AContext.Connection.Socket.ReadStream(AMemStream, PieceSize);
          finally
            AMemStream.Free;
          end; }
          AContext.Connection.Socket.ReadStream(AFileStream, PieceSize);
          Inc(SentAmount, PieceSize);

          if Assigned(ReceivingFileOnProgress) then
            ReceivingFileOnProgress(SentAmount);

          ExpectedResponse := IntToStr(i) + CRequestNextPiece_Middle;
          AContext.Connection.Socket.WriteLn(ExpectedResponse);
        end;

        CFinishFileTransfer:
        begin
          AddToLog('..... processing last piece');
          AddToLog('..... ExpectedFileSize: ' + IntToStr(ExpectedFileSize) + '   SentAmount: ' + IntToStr(SentAmount));
          LastPieceSize := ExpectedFileSize - SentAmount;
          ReceivedLastPieceSize := AContext.Connection.Socket.ReadUInt32(False);

          if LastPieceSize <> ReceivedLastPieceSize then
            AddToLog('..... transfer out of sync at last piece.');

          AddToLog('..... LastPieceSize: ' + IntToStr(LastPieceSize) + '   ReceivedLastPieceSize: ' + IntToStr(ReceivedLastPieceSize));

          if LastPieceSize > 0 then
          begin
            PieceIndex := AContext.Connection.Socket.ReadUInt32(False);
            AddToLog('..... last piece index: ' + IntToStr(PieceIndex));

            AContext.Connection.Socket.ReadStream(AFileStream, LastPieceSize);
            Inc(SentAmount, LastPieceSize);

            if Assigned(ReceivingFileOnProgress) then
              ReceivingFileOnProgress(SentAmount);
          end;

          ExpectedResponse := CRequestNextPiece_End;
          AContext.Connection.Socket.WriteLn(ExpectedResponse);
        end; //CFinishFileTransfer

        else
          AddToLog('..... Unknown piece state: ' + IntToStr(PieceState));
      end; //case PieceState
    end;  //for

    AddToLog('..... Transfer done');
    AddToLog('');
  finally
    AFileStream.Free;
  end;
end;

end.

