unit FileToPICServerMainForm;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  ComCtrls, IdTCPServer, IdCustomTCPServer, IdContext, IdSync;

type
  TSyncMsg = class(TIdSync)
  private
    FMessage: string;
  protected
    procedure DoSynchronize; override;
  end;


  TSyncBeat = class(TIdSync)
  protected
    procedure DoSynchronize; override;
  end;


  TSyncFilePosition = class(TIdSync)
  private
    FPosition: DWord;
  protected
    procedure DoSynchronize; override;
  end;


  TSyncFileSize = class(TIdSync)
  private
    FFileSize: DWord;
  protected
    procedure DoSynchronize; override;
  end;

  { TfrmFileToPICServerMain }

  TfrmFileToPICServerMain = class(TForm)
    chkActive: TCheckBox;
    IdTCPServer1: TIdTCPServer;
    lbePort: TLabeledEdit;
    memLog: TMemo;
    pnlBeat: TPanel;
    prbFileProgress: TProgressBar;
    tmrStartup: TTimer;
    tmrHideBeat: TTimer;
    procedure chkActiveChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure IdTCPServer1Connect(AContext: TIdContext);
    procedure IdTCPServer1Disconnect(AContext: TIdContext);
    procedure IdTCPServer1Execute(AContext: TIdContext);
    procedure tmrHideBeatTimer(Sender: TObject);
    procedure tmrStartupTimer(Sender: TObject);
  private
    FMainThreadID: TThreadID;
    FBaseDir: string;

    procedure DisconnectFromServer;

    procedure AddLineToLog(s: string);
    procedure AddLineToLogFromThread(s: string);

    procedure ReceiveFileOnProgress(APosition: DWord);
    procedure ReceiveFileOnProgressFromThread(APosition: DWord);

    procedure ReceivingFileSize(FileSize: DWord);
    procedure ReceivingFileSizeFromThread(FileSize: DWord);

    procedure ShowBeat;
    procedure ShowBeatFromThread;
  public

  end;

var
  frmFileToPICServerMain: TfrmFileToPICServerMain;

implementation

{$R *.frm}

uses
  FileToPICConsts, FileToPICReceiveFile;

procedure TSyncMsg.DoSynchronize;
begin
  frmFileToPICServerMain.AddLineToLog(FMessage);
end;


procedure TSyncBeat.DoSynchronize;
begin
  frmFileToPICServerMain.ShowBeat;
end;


procedure TSyncFilePosition.DoSynchronize;
begin
  frmFileToPICServerMain.ReceiveFileOnProgress(FPosition);
end;


procedure TSyncFileSize.DoSynchronize;
begin
  frmFileToPICServerMain.ReceivingFileSize(FFileSize);
end;


{ TfrmFileToPICServerMain }

procedure TfrmFileToPICServerMain.FormCreate(Sender: TObject);
begin
  AddLineToLog('Application startup');
  FMainThreadID := ThreadID;
  FBaseDir := ExtractFilePath(ParamStr(0)) + 'Transfer';
  tmrStartup.Enabled := True;
end;


procedure TfrmFileToPICServerMain.IdTCPServer1Connect(AContext: TIdContext);
begin
  AddLineToLogFromThread('Connected: ' + AContext.Connection.Socket.Binding.PeerIP +
                         '   FMainThreadID = ' + IntToStr(FMainThreadID) +
                         '   ThreadID = ' + IntToStr(ThreadID));
end;


procedure TfrmFileToPICServerMain.IdTCPServer1Disconnect(AContext: TIdContext);
begin
  AddLineToLogFromThread('Disconnected: ' + AContext.Connection.Socket.Binding.PeerIP +
                         '   FMainThreadID = ' + IntToStr(FMainThreadID) +
                         '   ThreadID = ' + IntToStr(ThreadID));
end;


procedure TfrmFileToPICServerMain.IdTCPServer1Execute(AContext: TIdContext);
var
  Cmd: string;
begin
  Cmd := AContext.Connection.Socket.ReadLn(#13#10, 5000);

  case Cmd of
    CSendBeatCmd: ShowBeatFromThread;
    CDisconnectCmd: AddLineToLogFromThread('Client is requesting disconnection...');
    CSendFileToSrvCmd: ReceiveFileFromClient(FBaseDir, AContext, @AddLineToLogFromThread, @ReceiveFileOnProgressFromThread, @ReceivingFileSizeFromThread);
  end;

  Sleep(1);
  //AddLineToLogFromThread('iteration');
end;


procedure TfrmFileToPICServerMain.tmrHideBeatTimer(Sender: TObject);
begin
  tmrHideBeat.Enabled := False;
  pnlBeat.Color := clGreen;
  pnlBeat.Repaint;
end;


procedure TfrmFileToPICServerMain.tmrStartupTimer(Sender: TObject);
begin
  tmrStartup.Enabled := False;

  if not DirectoryExists(FBaseDir) then
    CreateDir(FBaseDir); //doesn't work recursively
end;


procedure TfrmFileToPICServerMain.DisconnectFromServer;  //see StackOverflow question 11213571
var
  Context: TIdContext;
  i: Integer;
  AList: TList;
begin
  if IdTCPServer1.Active then
  begin
    AList := IdTCPServer1.Contexts.LockList;
    try
      for i := AList.Count - 1 downto 0 do
      begin
        Context := TIdContext(AList.Items[i]);
        if Context <> nil then
        begin
          Context.Connection.IOHandler.WriteBufferClear;
          Context.Connection.IOHandler.InputBuffer.Clear;
          Context.Connection.IOHandler.Close;
          if Context.Connection.Connected then
            Context.Connection.Disconnect(True);
        end;
      end;
    finally
      IdTCPServer1.Contexts.UnlockList;
    end;
  end;

  if IdTCPServer1.Active then  //check again
    IdTCPServer1.Active := False;
end;


procedure TfrmFileToPICServerMain.chkActiveChange(Sender: TObject);
begin
  if chkActive.Checked then
  begin
    IdTCPServer1.DefaultPort := StrToIntDef(lbePort.Text, 7000);
    try
      IdTCPServer1.Active := True;
    except
      on E: Exception do
        AddLineToLog('Can''t listen on port ' + lbePort.Text + '. ' + E.Message);
    end;
  end
  else
    DisconnectFromServer;
end;


procedure TfrmFileToPICServerMain.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
  DisconnectFromServer;
end;


procedure TfrmFileToPICServerMain.AddLineToLog(s: string);
var
  NowStr: string;
begin
  NowStr := DateTimeToStr(Now);
  s := StringReplace(s, #0, '#0', [rfReplaceAll]);
  s := StringReplace(s, #13#10, '  ', [rfReplaceAll]);
  s := StringReplace(s, #10, '  ', [rfReplaceAll]);

  memLog.Lines.BeginUpdate;
  try
    memLog.Lines.Add(NowStr + '  ' + s);
    memLog.SelStart := Length(memLog.Lines.Text) - 2;
    memLog.SelLength := Length(memLog.Lines.Text) - 1;
  finally
    memLog.Lines.EndUpdate;
  end;
end;


procedure TfrmFileToPICServerMain.AddLineToLogFromThread(s: string);
var
  SyncObj: TSyncMsg;
begin
  SyncObj := TSyncMsg.Create;
  try
    SyncObj.FMessage := s;
    SyncObj.Synchronize;
  finally
    SyncObj.Free;
  end;
end;


procedure TfrmFileToPICServerMain.ShowBeat;
begin
  pnlBeat.Color := clLime;
  pnlBeat.Repaint;
  tmrHideBeat.Enabled := True;
end;


procedure TfrmFileToPICServerMain.ShowBeatFromThread;
var
  SyncObj: TSyncBeat;
begin
  SyncObj := TSyncBeat.Create;
  try
    SyncObj.Synchronize;
  finally
    SyncObj.Free;
  end;
end;


procedure TfrmFileToPICServerMain.ReceiveFileOnProgress(APosition: DWord);
begin
  prbFileProgress.Position := APosition;
end;


procedure TfrmFileToPICServerMain.ReceiveFileOnProgressFromThread(APosition: DWord);
var
  SyncObj: TSyncFilePosition;
begin
  SyncObj := TSyncFilePosition.Create;
  try
    SyncObj.FPosition := APosition;
    SyncObj.Synchronize;
  finally
    SyncObj.Free;
  end;
end;


procedure TfrmFileToPICServerMain.ReceivingFileSize(FileSize: DWord);
begin
  prbFileProgress.Max := FileSize;
end;


procedure TfrmFileToPICServerMain.ReceivingFileSizeFromThread(FileSize: DWord);
var
  SyncObj: TSyncFileSize;
begin
  SyncObj := TSyncFileSize.Create;
  try
    SyncObj.FFileSize := FileSize;
    SyncObj.Synchronize;
  finally
    SyncObj.Free;
  end;
end;

end.

