unit DynTFTHandlers;

{$IFNDEF IsMCU}
  {$DEFINE IsDesktop}
{$ENDIF}

{$IFDEF IsDesktop}
interface
{$ENDIF}

uses
  {$IFDEF UseSmallMM}
    DynTFTSmallMM,
  {$ELSE}
    {$IFDEF IsDesktop}
      MemManager,
    {$ENDIF}  
  {$ENDIF} //this must be the first unit, at least in Delphi, because it exports GetMem.

  DynTFTTypes, DynTFTConsts, DynTFTUtils, DynTFTBaseDrawing, DynTFTControls,
  DynTFTGUIObjects,


//<DynTFTComponents>
  DynTFTButton,
  DynTFTArrowButton,
  //DynTFTPanel,
  //DynTFTCheckBox,
  DynTFTScrollBar,
  DynTFTItems,
  DynTFTListBox,
  DynTFTLabel,
  DynTFTRadioButton,
  DynTFTRadioGroup,
  DynTFTTabButton,
  DynTFTPageControl,
  DynTFTEdit,
  //DynTFTKeyButton,
  //DynTFTVirtualKeyboard,
  //DynTFTComboBox,
  DynTFTTrackBar,
  DynTFTProgressBar,
  DynTFTMessageBox
//<EndOfDynTFTComponents> - Do not remove or modify this line!

  {$IFDEF IsDesktop}
    ,SysUtils, Forms
    {$IFDEF DynTFTFontSupport}, DynTFTFonts {$ENDIF}
  {$ENDIF}

  {$IFNDEF UserTFTCommands}
    {$IFDEF IsDesktop} , TFT {$ENDIF}
  {$ELSE}
    , {$I UserDrawingUnits.inc}
  {$ENDIF}

  {$I DynTFTHandlersAdditionalUnits.inc}
  ;
  
  {$IFDEF UseHandlersAdditionalCodeInInterface}
    {$I DynTFTHandlersAdditionalCodeInInterface.inc} //Can be used to specify external functions, vars, consts
  {$ENDIF}

//CodegenSym:GroupsBegin
//CodegenSym:GroupsEnd

procedure PageControl1_OnChange(Sender: PPtrRec); //CodegenSym:header
procedure lstClientStatus_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:header
procedure trbBackLight_OnTrackBarChange(Sender: PPtrRec); //CodegenSym:header
procedure btnSetIP_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:header

//CodegenSym:AllBinHandlersBegin

{$IFDEF RTTIREG}

  procedure ExecuteDesignRTTIInstructions_Create(Filter: Byte);   //If a compiler complains about missing implementation, probably regenerating the output files from DynTFTCodeGen, will fix the issue.
  procedure ExecuteDesignRTTIInstructions_Destroy(Filter: Byte);  //If a compiler complains about missing implementation, probably regenerating the output files from DynTFTCodeGen, will fix the issue.

  {$IFDEF IsDesktop} {$IFDEF DesktopApp_FP} // Profile: "DesktopApp_FP"
      var
        AllBinHandlers: array[0..2] of TPtr;

      procedure UpdateAllBinHandlerAddressArray;

    procedure LinkHandlers;
  {$ENDIF} {$ENDIF} // IsDesktop

  {$IFDEF IsMCU} {$IFDEF PIC32App} // Profile: "PIC32App"
      var
        AllBinHandlers: array[0..2] of TPtr;

      procedure UpdateAllBinHandlerAddressArray;

    procedure LinkHandlers;
  {$ENDIF} {$ENDIF} // IsMCU

  {$IFDEF IsDesktop}
      var
        AllBinHandlersStr: array[0..2] of string;
        AllBinHandlersAddresses: array[0..2] of TPtr;

        AllBinIdentifiersStr: array[0..0] of string;  // No identifiers found. Using a dummy entry.
        AllBinIdentifiersAddresses: array[0..0] of TPtr;  // No identifiers found. Using a dummy entry.

      procedure UpdateAllBinHandlerStrArray;
  {$ENDIF}
{$ENDIF} // RTTIREG

//CodegenSym:AllBinHandlersEnd

implementation

{$I DynTFTHandlersAdditionalCode.inc}

//CodegenSym:UpdateBinHandlersProcBegin
{$IFDEF RTTIREG}
  {$IFDEF IsDesktop} {$IFDEF DesktopApp_FP} // Profile: "DesktopApp_FP"
      procedure UpdateAllBinHandlerAddressArray;
      begin
        AllBinHandlers[0] := TPtr(@lstClientStatus_OnMouseUpUser);
        AllBinHandlers[1] := TPtr(@trbBackLight_OnTrackBarChange);
        AllBinHandlers[2] := TPtr(@btnSetIP_OnMouseUpUser);
      end;

    procedure ExecuteDesignRTTIInstructions_Create(Filter: Byte);
    var
      GetDataCallback: {$IFDEF IsDesktop} TDynTFTExecRTTIInstructionCallback {$ELSE} PDynTFTExecRTTIInstructionCallback {$ENDIF};
    begin
      GetDataCallback := nil;  // no data provider callback is used. ExecuteRTTIInstructions expects that "C_RTTI_CreateInstructionData" holds all instructions.
      ExecuteRTTIInstructions(@C_RTTI_CreateInstructionData, @CAllCreatedBinComponents, @AllBinHandlers, Filter, GetDataCallback);
    end;

    procedure ExecuteDesignRTTIInstructions_Destroy(Filter: Byte);
    var
      GetDataCallback: {$IFDEF IsDesktop} TDynTFTExecRTTIInstructionCallback {$ELSE} PDynTFTExecRTTIInstructionCallback {$ENDIF};
    begin
      GetDataCallback := nil;  // no data provider callback is used. ExecuteRTTIInstructions expects that "C_RTTI_DestroyInstructionData" holds all instructions.
      ExecuteRTTIInstructions(@C_RTTI_DestroyInstructionData, @CAllCreatedBinComponents, @AllBinHandlers, Filter, GetDataCallback);
    end;

    procedure LinkHandlers;
    {$IFDEF IsMCU}
      var
        Dummy: DWord; volatile;
    {$ENDIF}
    begin
      SetFuncCall(lstClientStatus_OnMouseUpUser);
      SetFuncCall(trbBackLight_OnTrackBarChange);
      SetFuncCall(btnSetIP_OnMouseUpUser);
    end;
  {$ENDIF} {$ENDIF} // IsDesktop {$IFDEF DesktopApp_FP}

  {$IFDEF IsMCU} {$IFDEF PIC32App} // Profile: "PIC32App"
      procedure UpdateAllBinHandlerAddressArray;
      begin
        AllBinHandlers[0] := TPtr(@lstClientStatus_OnMouseUpUser);
        AllBinHandlers[1] := TPtr(@trbBackLight_OnTrackBarChange);
        AllBinHandlers[2] := TPtr(@btnSetIP_OnMouseUpUser);
      end;

    procedure ExecuteDesignRTTIInstructions_Create(Filter: Byte);
    var
      GetDataCallback: {$IFDEF IsDesktop} TDynTFTExecRTTIInstructionCallback {$ELSE} PDynTFTExecRTTIInstructionCallback {$ENDIF};
    begin
      GetDataCallback := nil;  // no data provider callback is used. ExecuteRTTIInstructions expects that "C_RTTI_CreateInstructionData" holds all instructions.
      ExecuteRTTIInstructions(@C_RTTI_CreateInstructionData, @CAllCreatedBinComponents, @AllBinHandlers, Filter, GetDataCallback);
    end;

    procedure ExecuteDesignRTTIInstructions_Destroy(Filter: Byte);
    var
      GetDataCallback: {$IFDEF IsDesktop} TDynTFTExecRTTIInstructionCallback {$ELSE} PDynTFTExecRTTIInstructionCallback {$ENDIF};
    begin
      GetDataCallback := nil;  // no data provider callback is used. ExecuteRTTIInstructions expects that "C_RTTI_DestroyInstructionData" holds all instructions.
      ExecuteRTTIInstructions(@C_RTTI_DestroyInstructionData, @CAllCreatedBinComponents, @AllBinHandlers, Filter, GetDataCallback);
    end;

    procedure LinkHandlers;
    {$IFDEF IsMCU}
      var
        Dummy: DWord; volatile;
    {$ENDIF}
    begin
      SetFuncCall(lstClientStatus_OnMouseUpUser);
      SetFuncCall(trbBackLight_OnTrackBarChange);
      SetFuncCall(btnSetIP_OnMouseUpUser);
    end;
  {$ENDIF} {$ENDIF} // IsMCU {$IFDEF PIC32App}


  {$IFDEF IsDesktop}
      procedure UpdateAllBinHandlerStrArray;
      begin
        AllBinHandlersStr[0] := '_lstClientStatus_OnMouseUpUser';
        AllBinHandlersStr[1] := '_trbBackLight_OnTrackBarChange';
        AllBinHandlersStr[2] := '_btnSetIP_OnMouseUpUser';

        AllBinHandlersAddresses[0] := TPtr(@lstClientStatus_OnMouseUpUser);
        AllBinHandlersAddresses[1] := TPtr(@trbBackLight_OnTrackBarChange);
        AllBinHandlersAddresses[2] := TPtr(@btnSetIP_OnMouseUpUser);

        AllBinIdentifiersStr[0] := 'DummyProc';
        AllBinIdentifiersAddresses[0] := TPtr(nil);
      end;
  {$ENDIF}
{$ENDIF} // RTTIREG

//CodegenSym:UpdateBinHandlersProcEnd

//CodegenSym:CreationGroups

//CodegenSym:HandlersImplementation

procedure PageControl1_OnChange(Sender: PPtrRec); //CodegenSym:handler
var
  i: Integer;
begin //CodegenSym:handler:begin
  {$IFDEF IsDesktop}
    DynTFT_DebugConsole('--------------------------- changing tab to ' + IntToStr(PageControl1^.ActiveIndex + 1));
  {$ENDIF}

  DynTFT_Set_Pen(DynTFTAllComponentsContainer[PageControl1^.ActiveIndex + 1].ScreenColor, 1);
  DynTFT_Set_Brush(1, DynTFTAllComponentsContainer[PageControl1^.ActiveIndex + 1].ScreenColor, 0, 0, 0, 0);
  DynTFT_Rectangle(0, 28, TFT_DISP_WIDTH, TFT_DISP_HEIGHT);

  for i := 1 to PageControl1^.PageCount {- 1} do
  begin
    DynTFTAllComponentsContainer[i].Active := PageControl1^.ActiveIndex = i - 1;
    if DynTFTAllComponentsContainer[i].Active then
      DynTFTRepaintScreenComponents(i, CREPAINTONSTARTUP, nil);
  end;
end; //CodegenSym:handler:end


procedure lstClientStatus_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:handler
var
  i: Integer;
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    DisplaySocketStatusByIndex(lstClientStatus^.Items^.ItemIndex, socket_Intern[lstClientStatus^.Items^.ItemIndex].state, True);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure trbBackLight_OnTrackBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    TFT_Set_DBC_SSD1963(trbBackLight^.Position);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure btnSetIP_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    ManualIPAndMACConfiguration(rdgrpIPAddr^.ItemIndex);
    Net_Ethernet_Intern_Init(@myMacAddr, @myIpAddr, Net_Eth_Int_AUTO_NEGOTIATION or Net_Eth_Int_DEFAULT_MAC or Net_Eth_Int_SPEED_100 or Net_Eth_Int_FULLDUPLEX);
  {$ENDIF}
end; //CodegenSym:handler:end


end.
